"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.lambdaApiAuth = void 0;
const axios_1 = __importDefault(require("axios"));
const config_1 = require("./config");
const getAccount = (req, headers) => __awaiter(void 0, void 0, void 0, function* () {
    const res = yield axios_1.default({
        method: 'GET',
        url: `${config_1.config.PRODUCT_API_HOST}/account`,
        headers: headers,
    });
    // Enrich request object with account data
    req.account = res.data;
});
const getWorkspace = (req, headers, workspaceId) => __awaiter(void 0, void 0, void 0, function* () {
    const res = yield axios_1.default({
        method: 'GET',
        url: `${config_1.config.PRODUCT_API_HOST}/account/workspaces/${workspaceId}`,
        headers: headers,
    });
    // Enrich request object with workspace data
    req.workspace = res.data;
});
const getDomain = (req, headers, domainId) => __awaiter(void 0, void 0, void 0, function* () {
    const res = yield axios_1.default({
        method: 'GET',
        url: `${config_1.config.PRODUCT_API_HOST}/account/domains/${domainId}`,
        headers: headers,
    });
    // Enrich request object with domain data
    req.domain = res.data;
});
/**
 * Adds an authorization middleware layer with optional validation of workspace and/or domain.
 * @param {Request} req
 * @param {Response} res
 * @param {NextFunction} next
 * @return {Promise<void>}
 */
exports.lambdaApiAuth = (req, res, next) => __awaiter(void 0, void 0, void 0, function* () {
    const headers = {
        Accept: 'application/json'
    };
    // Verify authentication (apikey or oauth token)
    if (req.headers['apikey']) {
        headers['apikey'] = req.headers['apikey'];
    }
    else if (req.headers['authorization']) {
        headers['Authorization'] = req.headers['authorization'];
    }
    else {
        return res.status(401).send('Unauthorized');
    }
    const promises = [
        getAccount(req, headers),
    ];
    // Workspace validation: try extracting workspace from query params or headers
    if (config_1.config.VALIDATE_WORKSPACE) {
        if (req.query.workspaceId || req.headers.workspace) {
            const workspaceId = req.query.workspaceId || req.headers.workspace;
            promises.push(getWorkspace(req, headers, workspaceId));
        }
    }
    // Domain validation: try extracting domain from query params
    if (config_1.config.VALIDATE_DOMAIN) {
        if (req.query.domainId) {
            promises.push(getDomain(req, headers, req.query.domainId));
        }
    }
    try {
        yield Promise.all(promises);
    }
    catch (err) {
        if (err.isAxiosError && err.response && err.response.status < 500) {
            console.error(err);
            return res.status(err.response.status).send(err.response.data || '');
        }
        throw err;
    }
    return next();
});
